/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Bidouilleur PIC12F683 - exemple de Diviseur PIC 
  lectro-Bidouilleur, Novembre 2017


	TARGET device: PIC12F683
	Compilateur: SourceBoost's BoostC pour PIC
	IDE: SourceBoost IDE
*/
// INSTRUCTIONS AU COMPILATEUR
#include <PIC12F683.h>
#include <system.h>

// CONFIGURATION BYTES				// Configurer pour l'horloge externe!
#pragma DATA _CONFIG, _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _EC_OSC  
//#pragma CLOCK_FREQ 1000000		// Tells the compiler what clock frequency is used. Important for delay functions

void main()
{
	// Initialization stage
	intcon 		= 	0b00000000; 		// disable interrupts.	
	ioc    		=	0b00000000;		// disable gpio interrupts on change
	pie1   		=	0b00000000; 		// disable various interrupts
	option_reg 	= 	0b00001111;		// weak pullups global disable, prescaler assigned to WDT and set to 1:128
	wpu    		=	0b00000000;		// individual weak pullups disabled
	osccon		=	0b11100000; 		// select External oscillator	
	t1con		=	0b00110000;		// disable timer1 and set prescaler div. by 8 
	t2con		=	0b01111010;		// disable timer2 and set prescaler div. by 16 and postscaler div. by 16 
	adcon0		=	0b01000000;		// disable A/D converter
	ansel		=	0b00000010;		// Enable analog assignment to pin CIN- (GP1)
	cmcon0		=	0b00000100;		// enable comparator, select CIN- as inverting input, use internal reference, invert output.
	cmcon1 		=	0b00000000;		// comparator gating timer 1 asynchronously
	vrcon 		=	0b10100010;		// Enable internal comparator Vref, use low range, set Vref to 0.42V 
	wdtcon		=	0b00010100;		// Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
	gpio 		=	0b00000000;		// Disable PTT (low), lows on other Port pins.
	trisio 		=	0b11111110;		// GPIO first disabled, but assigned as follows:    
										//	5-Entre/Sortie disponible
										//	4-Entre/Sortie disponible
										//	3-Entre uniquement, disponible
										//	2-Entre/Sortie disponible
										//	1-Entre/Sortie disponible
										//	0-Sortie du diviseur PIC

unsigned char d1, d2;				// les deux compteur de division utiliss
asm
{
//  Attention: 
//   - La frquence d'entre du signal est divise par 4  l'interne avant de cadencer le code.
//   - Chaque passage de la boucle entire reprsente un demi-cycle du signal de sortie, d'o le facteur de 2.

//  Diviseur par 100000 dsir.
//   - Pour la boucle entire, 100000 / 4 / 2 = 12500:  Nombre entier, donc divisible par 8.
//   - Chaque itration de la boucle prend 12 cycles.
//   - 12500 pas divisible par 12. 
//   - mais 12480 divisible par 12.
//   - Donc 12500 = 12480 + 20 = (104 * 10 * 12 cycles) + 20

Init:   movlw	104					; charger 104			
		movwf	_d1					;   dans d1
		movlw	10					; charger 10
		movwf	_d2					;   dans d2

  ; Toutes les itrations dans cette boucle comportent 12 cycles, sauf la dernire qui en comporte 20 de plus.
Boucle:	decfsz	_d1, 1				; 1 cycle, ou 2 cycles lorsque la variable devient zro				
		goto	Saut1				; 2 cycles
		movlw	104					; 1 cycle		ici d1=0, r-initialiser d1 avec la valeur 104
		movwf	_d1					; 1 cycle
		decfsz	_d2, 1				; 1 cycle, ou 2 cycles lorsque la variable devient zro
		goto	Saut2				; 2 cycles	 
		movlw	10					; 1 cycle		ici d2=0, r-initialiser d1 avec la valeur 10
		movwf	_d2					; 1 cycle
		movlw 0x01 					; 1 cycle      	Basculer la broche GP0 											
		xorwf _gpio, 1 				; 1 cycle 		  en utilisant XORWF				

		nop							; 1 cycle     
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle     20 instructions additionnelles pour complter  12500
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle

		goto	Boucle				; 2 cycles

Saut1:	nop
		nop							; 1 cycle
		nop							; 1 cycle
		nop							; 1 cycle
Saut2:	nop							; 1 cycle		pour complter  12 cycles
		nop							; 1 cycle		
		nop							; 1 cycle
		goto	Boucle				; 2 cycles
  }
}
